#!/bin/sh
#
# @(#) Script to update disk drive firmware.
#
# Copyright (c) 2010-2017 by Dell Inc.
#
# All rights reserved.  This software may not be copied, disclosed,
# transferred, or used except in accordance with a license granted
# by Dell Inc.  This software embodies proprietary information
# and trade secrets of Dell Inc.
#
#
VERSION=11.0
# Source our library of functions
. ./lib/raidLib.sh
. ./lib/eqlDiskLib.sh
#
#
# set some globals.
#  init and update the progress count
g_currentProgress=0
TDLI=""
# set  the minimum version requirement.
reqMajVer=6
reqMinVer=0
reqMaintVer=0
# ------------------------------------------
#       Function:     fn_GSBfix()
#        This routine will check the GSB value to verify that it is set to "E" (0x45).
#        If the GSB is "U" an attempt will be made to change it to "E". If "C" (ox42)
#        we will exit. SATA drives are excluded as are drives on the blacklist which
#        lack support for GSB. See Bug 72680 GSB -  Support for Single Part Number
#        firmware changes for additional detail.
#                
#       Inputs:
#        driveID - the drive ID to check for the issue
#        driveModel - the drive model to be validated for GSB support
#
#       Outputs/Side Effects:
#        echo/log any important information detected for the request drive ID
#
#       Returns:
#            0
#            1
#
fn_GSBfix()
{
    driveID=$1
    driveModel=$2
    U="GSB Value is 0x55"
    E="GSB Value is 0x45"
    C="GSB Value is 0x43"
    noGSBDriveModel=`awk /${driveModel}/ ./lib/gsbBlackList.lst`
    SATAFound=`diskview -i $driveID | awk 'BEGIN{IGNORECASE=1} /SATA/'`
    gsbByte=`echo -e "$driveID\nb\nq\nq\n" | scsitool | awk /'GSB Value is'/`
    # check we are SATA
    if [ ! -z "${SATAFound}" ]; then
	fn_Log "      Disk type is SATA for DriveID ${driveID}, skipping GSB check" "T"
	    else
            # check if the drive supports GSB modification
	    if [ ! -z "${noGSBDriveModel}" ]; then
            fn_Log "      Drive Model ${driveModel} does not support GSB modification" "T"
                else
                # Check for Compellent GSB
                if [ "$gsbByte" = "$C" ]; then
                fn_Log "      An unexpected GSB value was detected on DriveID ${driveID}" "T"
                fn_Cleanup 1
                    else
                    if [ "$gsbByte" = "$U" ]; then
	            # Changing the GSB from "U" 0x55 to "E" 0x45
                    echo -e "$driveID\nB\nq\nq\n" | scsitool > /dev/null 2>&1
                    fn_Log "      GSB value changed to E for DriveID ${driveID}" "T"
                    fi
                    # Verify the new GSB value
                    gsbByte=`echo -e "$driveID\nb\nq\nq\n" | scsitool | awk /'GSB Value is'/`
                    if [ "$gsbByte" != "$E" ]; then
                    fn_Log "      Failed to change the GSB value to E for DriveID ${driveID}" "T"
                    fn_Cleanup 1
                    else
                    return 0
                    fi
		fi
            fi
    fi		
}

# ------------------------------------------
#       Function:     fn_notifyGui()
#               This routine logs a short event with message in the eqllogger when the drive firmware update
#               is being run in gui mode and echo's the same message to stderr.
#
#       Inputs:
#        _eventMsg_  string tied to generic event id 35.2.2.
#
#       Outputs/Side Effects:
#         logs and event in the eqllogger and send the same message to stderr ONLY when guiMode(a global) is detected.
#
#       Returns:
#            none
#
fn_notifyGui()
{
    eventMsg="$1"
    if [ ! -z "${g_guiMode}" ]; then
        echo "`date '+%H:%M:%S'` ${eventMsg}" >&2
        logevent 35.2.2 "${eventMsg}" 2> /dev/null
    fi
}

#
#
#
#
# ------------------------------------------
#       Function:     fn_initProgressCalc()
#               This routine initializes the TDLI information used to calculate the amount of
#       progress an interative tasks should take.
#         _TDLI_ This is a colon separated string representing the
#                 T:ime percentage of a task will consume
#                 D:ivisor is the granularity to break the Time percentage into
#                 L:left over fraction of time from T divided_by D
#                 I:ncrement is the amount that would be added in a call to fn_progressMsg()
#
#       Inputs:
#        _percentExpenseForTask_   the percentage of a process that this task should take
#
#       Outputs/Side Effects:
#         _TDLI_ The contrived string is echoed to stdout
#
#       Returns:
#            _none_
#
fn_initProgressCalc()
{
    # Initialize the weighted Progress costs var TDLI
    percentExpenseForTask=$1
    # next init the progress calc for
    totalDrives=`diskview -I s | awk '{print $2}' | awk 'BEGIN{count=0}/[0-9]+/{count++}END{print count}'`
    TDLI="${percentExpenseForTask}:${totalDrives}:0:0"
    echo $TDLI
    return
}
#
# ------------------------------------------
#       Function:     fn_incrementProgressCalc()
#               This routine calculates the amount of increment to bump the guiMode progress message.
#
#       Inputs:
#         _TDLI_ This is a colon separated string representing the
#                 T:ime percentage of a task will consume
#                 D:ivisor is the granularity to break the Time percentage into
#                 L:left over fraction of time from T divided_by D
#                 I:ncrement is the amount that would be added in a call to fn_progressMsg()
#
#       Outputs/Side Effects:
#         _TDLI_ The recalculated contrived string is echoed to stdout
#
#       Returns:
#            _g_currentProgress_  current progress value is returned
#

fn_incrementProgressCalc()
{
    TDLI=$1
    # take the input arg and perform the calculation
    TDLI=`echo $TDLI | awk -f alib/progressCalc.awk`
    incr=`echo $TDLI | awk -F ":" '{print $4}'`
    if [ $incr -ne 0 ]; then
        fn_progressMsg ${incr}
    fi
    echo $TDLI
    return $g_currentProgress
}
#
# ------------------------------------------
#       Function:     fn_DetectKD03Markers()
#               This routine detect whether a given driveID is seagate drive with the KD03 state issue.
# There are 3 criteria that need to be present to confirm that the drive has the KD03 State issue.
# 1.) left justified FWRev
# 2.) WS is not set
# 3.) the requested drive's serial number in the big list
#
#       Inputs:
#             driveID   the drive ID to check for the issue
#
#       Outputs/Side Effects:
#      - echo/log any important information detected for the request drive ID
#
#       Returns:
#       0  We failed to detect ANY markers for the KD03 corruption state issue with the requested drive ID. Drive is ok.
#       1  We successfully detected ALL drive markers indicating the KD03 state issues with the requested drive ID...
#          Muskietool processing required
#       2  We successfully detected  the first 2 drive markers but didn't match the SN...Perfecta
#          Muskietool processing required
#       3  We matched either of the first 2 markers but NOT both. This should never be the case...Problemo
#
fn_DetectKD03Markers()
{
    driveID=$1

    perfecta=0
    trifecta=0
    #
    #  Check #1
    fn_Log "    Special processing for KD03" "T"
    fn_Log "    Detecting KD03 Markers" "T"
    fn_Log ""
    problemKd03=`diskview -i $driveID | awk '/=KD03/'`
    if [ ! -z "${problemKd03}" ]; then
        fn_Log "    MARKER #1 FOUND: Drive  ${driveID} Left justified  KD03 string found . " "T"
        perfecta=`echo $perfecta | awk '{ print $0 +1}'`
    fi
    # check #2
    problemKd03=`diskview -i $driveID | awk '/WS\|/'`
    if [  -z "${problemKd03}" ]; then
        fn_Log "    MARKER #2 FOUND: Drive  ${driveID}  WS setting correctly NOT found on drive .  " "T"
        perfecta=`echo $perfecta | awk '{ print $0 +1}'`
    fi
    # check #3
    driveSn=`fn_eqlDiskSerialNumber $driveID`
    problemKd03=`cat ./fwdsc/kd03SN.lst | awk -v SN=$driveSn  -f ./alib/driveSerialMatcher.awk`
    if [ ! -z "${problemKd03}" ]; then
        fn_Log "    MARKER #3 FOUND: Drive  ${driveID} serial number ${driveSn} found in the list of affected drives.  " "T"
        trifecta=`echo $perfecta | awk '{ print $0 +1}'`
    fi
    ###############################################
    #  check what we found and return appropriately
    #
    ###############################################
    #
    # Drive is not amp corrupted
    if [ $perfecta -eq 0 -a $trifecta -eq 0 ]; then
        fn_Log "    Drive ${driveID} NO CORE MARKERS WERE FOUND" "T"
        return 0
    fi
    # Drive is not amp corrupted however we have a false positive on the SN
    if [ $perfecta -eq 0 -a $trifecta -eq 3 ]; then
        fn_Log "    Drive ${driveID} JUST SN MARKER WAS FOUND" "T"
        return 0
    fi
    # We got a match of all 3 markers
    if [ $perfecta -eq 2 -a $trifecta -eq 3 ]; then
        fn_Log "    Drive ${driveID} ALL MARKERS FOUND." "T"
        return 1
    fi
    # We matched the first 2 markers but the SN.
    if [ $perfecta -eq 2 -a $trifecta -eq 0 ]; then
        fn_Log "    Drive ${driveID} SN MARKER WAS NOT FOUND. First 2 markers were found." "T"
        return 2
    fi
    # We matched just one ofthe first 2 markers.
    if [ $perfecta -eq 1  ]; then
        fn_Log "    Drive ${driveID} Only one of the first two markers was found. They should either both be present or both be absent" "T"
        return 3
    fi
}

# ------------------------------------------
#       Function:     fn_ProcessSpecialCases()
#               This routine will process any special cases we may have when updating firmware
#       on drives.
#
#       Inputs:
#             driveID   the drive ID to check for the issue
#
#       Outputs/Side Effects:
#      - echo/log any important information detected for the request drive ID
#
#       Returns:
#       0  We successfully processed any special cases or there was nothing to do for this driveID
#       1  We failed to process any special cases.
#
fn_ProcessSpecialCases()
{
	driveID=$1
    ################################################
    # special case Number 1.... KDO3 AMP corruption
    ################################################
    kd03Found=`diskview -i $driveID |  awk 'BEGIN{IGNORECASE=1} /KD03/'`
    if [ ! -z "${kd03Found}" ]; then
        # HACK ALERT to handle SEAGATE issue with KD03 corrupted amp
        fn_DetectKD03Markers $driveID
        kd03State=$?
        # check the captured return for the worst case.
        if [ ${kd03State} -eq 3 ]; then
            fn_notifyGui "ABORTING:  We have an unknown state on Drive: ${driveID}"
            fn_Log "ABORTING:  We have an unknown state on Drive: ${driveID}"
            fn_Cleanup 1
        fi
        # handle these states with muskietool if possible.
        if [  ${kd03State} -eq 1  -o ${kd03State} -eq 2  ]; then
            fn_Log "    Legacy Version of firmware found on drive ${driveID}. Special pre-processing will be applied. " "T"
            muskieTool="./bin/muskietool"
            # muskietool is different in springbank(ie...V7)
            curVersion=`fn_CheckVersionMinimum 7 0 0`
            if [ $? -eq 0 ] ; then
                muskieTool="./bin/muskietoolV7"
            fi

            chmod 755 ${muskieTool}
            fn_Log "    ####  Running ${muskieTool} -R ${driveID} ####" "T"
            mOut=`${muskieTool} -R ${driveID}`
            if [ $? -ne 0 ]; then
                fn_notifyGui "ERROR:  ${muskieTool} failed on drive ${driveID}: ${mOut}"
                fn_Log "    ERROR:  ${muskieTool} failed on drive ${driveID}: ${mOut}" "T"
                return 1
            else
                return 0
            fi
        fi
    fi
    ################################################
    # special case Number 2.... TBD
    ################################################
	
    # We fell all the way thru.
    return 0
}

# ------------------------------------------
#       Function:     fn_Updatecnt()
#               This routine counts the drives that are "updateable";
#               a. The drive is online
#               b. It has a firmware version that we can update
#               c. Array firmware version meets minimum revision to update this drive
#
#
#       Inputs:
#               none
#
#       Outputs/Side Effects:
#      -  echo the updateable drive count
#      note: all other outputs will only go to the log file.
#
#       Returns:
#       0  successful
#       1  failure
#
fn_Updatecnt()
{
    updateableDriveCount=0
    oldVerField=5
    newVerField=4

    if [ ! -z "${g_guiMode}" ]; then
        TDLI=`fn_initProgressCalc  44`
    fi

    for driveID in `diskview -I s | awk '{print $2}' | awk '/[0-9]+/' `
    do
    # we don't use all of these just now but this at least will expose them as obvious for the future.

        driveModel=`fn_eqlDiskModelNumber $driveID`
        driveFwRev=`fn_eqlDiskRevisionNumber $driveID`
        driveStatus=`diskview -i $driveID | awk '/State:/ {print $7}'`

        if [ "$driveStatus" = "online" ] || [ "$driveStatus" = "Online" ]; then
            firmwareDescriptorFile="${driveModel}.dsc"
            if [ -f ./fwdsc/${firmwareDescriptorFile}  ] ; then
                # see if the new firmware available is different then the firmware we are using.(skip comment lines)
                availableFwRev=`cat ./fwdsc/${firmwareDescriptorFile} | awk -v WHICH_FIELD=$newVerField   -f ./alib/readFieldInCommentedFile.awk`
                # now check that it is an EQL firmware that we are coming from.
                eqlFwRevs=`cat ./fwdsc/${firmwareDescriptorFile} | awk -v WHICH_FIELD=$oldVerField   -f ./alib/readFieldInCommentedFile.awk`
                oneOfOurs=0
                for eqlFw in `echo $eqlFwRevs | awk '{ gsub(/,/, " ", $0) } END{ print $0 }'`; do
                    if [ "$driveFwRev" = "$eqlFw" ]; then
                        oneOfOurs=1
                    fi
                done
                # ok count them up.
                if [ "$driveFwRev" != "$availableFwRev" ] && [ $oneOfOurs -eq 1 ]; then
                    updateableDriveCount=`echo $updateableDriveCount | awk '{print $0 + 1}'`
                fi
                if [ -z ${availableFwRev} ]; then
                    return 1
                fi
            fi
        fi
        # should this be a gui mode update then we need to calc/increment progress here
        if [ ! -z "${g_guiMode}" ]; then
            TDLI=`fn_incrementProgressCalc "$TDLI"`
            g_currentProgress=$?
        fi
    done
    return 0
}
# ------------------------------------------
#       Function:     fn_UpdateDriveFirmware()
#               This routine updates all eligible online drives
#               a. The drive is online
#               b. It has a firmware version that we can update
#
#
#       Inputs:
#               _updateableDriveCount_  The number of drives to be updated
#
#       Outputs/Side Effects:
#      -  echo the updateable drive count
#
#       Returns:
#       0  successful
#       1  failure
#
fn_UpdateDriveFirmware()
{
    updateableDriveCount=$1

    oldVerField=5
    newVerField=4
    newBinField=3

    # Initialize the weighted Progress costs var TDLI
    if [ ! -z "${g_guiMode}" ]; then
        TDLI=`fn_initProgressCalc  48`
    fi

    for driveID in `diskview -I s | awk '{print $2}' | awk '/[0-9]+/' `
    do
    # we don't use all of these just now but this at least will expose them as obvious for the future.

        driveModel=`fn_eqlDiskModelNumber $driveID`
        driveFwRev=`fn_eqlDiskRevisionNumber $driveID`
        driveStatus=`diskview -i $driveID | awk 'BEGIN{IGNORECASE=1} /online/'`
		
        if [ ! -z "${driveStatus}"  ]; then
            firmwareDescriptorFile="${driveModel}.dsc"
            if [ -f ./fwdsc/${firmwareDescriptorFile}  ] ; then
                # see if the new firmware available is different then the firmware we are using.(skip comment lines)
                availableFwRev=`cat ./fwdsc/${firmwareDescriptorFile} | awk -v WHICH_FIELD=$newVerField   -f ./alib/readFieldInCommentedFile.awk`
                if [ "$driveFwRev" = "$availableFwRev" ]; then
                    fn_notifyGui "SKIPPING DriveID ${driveID}  It is already running ${driveFwRev} the appropriate firmware for Disk Model ${driveModel} "
                    fn_Log "    SKIPPING DriveID ${driveID}  It is already running ${driveFwRev} the appropriate firmware for Disk Model ${driveModel} "
                    fn_Log " "
                    # update the progress count
                    fn_progressMsg 1
                else
                    # now check that it is an EQL firmware that we are coming from.
                    eqlFwRevs=`cat ./fwdsc/${firmwareDescriptorFile} | awk -v WHICH_FIELD=$oldVerField   -f ./alib/readFieldInCommentedFile.awk`
                    oneOfOurs=0
                    for eqlFw in `echo $eqlFwRevs | awk '{ gsub(/,/, " ", $0) } END{ print $0 }'`;do
                        if [ "$driveFwRev" = "$eqlFw" ]; then
                            oneOfOurs=1
                        fi
                    done
                    if [ $oneOfOurs -eq 1 ]; then
                        fn_VerifyChecksums ./fwdsc/${firmwareDescriptorFile}
                        if [ $? -eq 0 ]; then
                            fwBin=`cat ./fwdsc/${firmwareDescriptorFile} | awk -v WHICH_FIELD=$newBinField   -f ./alib/readFieldInCommentedFile.awk`
                            if [ -z "${fwBin}" ]  || [ ! -f ./fwbin/${fwBin} ]; then
                                fn_notifyGui "ERROR: Unable to find the firmware binary file.   ( ./fwbin/${fwBin} )"
                                fn_Log "   ERROR: Unable to find the firmware binary file.   ( ./fwbin/${fwBin} )"
                                fn_Cleanup 1
                            fi
                            #We NOW check RAID every time thru the loop as the conservative way to proceed.
                            # Check overall health of Array
                            fn_Log " " "T"
                            rmsg=`fn_GetRaidStatus`
                            #PC thinking I might want to sleep here.
                            rstat=$?
                            # we will accept only a few RAID states present during a firmware upgrade.
                            if [ $rstat -ne 0 ] && [  $rstat -ne 2 ] ; then
                                fn_notifyGui  "${rmsg}"
                                fn_Log "${rmsg}"
                                fn_Log "Exiting with a status of 1"
                                fn_Cleanup 1
                            fi
                            # ok RAID is good  now check ANY for bad/failed drives
                            badFailed=`diskview -I s |  awk 'BEGIN{IGNORECASE=1} /bad/'`
                            if [ ! -z "${badFailed}"  ]; then
                                fn_notifyGui "Cannot proceed with  failed drives... ${badFailed}"
                                fn_Log "Cannot proceed with  failed drives... ${badFailed}"
                                fn_Log "Exiting with a status of 1"
                                fn_Cleanup 1
                            fi
                            fn_GetArrayHealthStatus
                            if [ $? -ne 0 ]; then
                                fn_Cleanup 1
                            fi
                            fn_ProcessSpecialCases $driveID
                            if [ $? -eq 0 ]; then
                                fn_Log "    UPGRADING DriveID ${driveID} firmware from ${driveFwRev} to ${availableFwRev} using ./fwbin/${fwBin}"
                                dtMsg=`disktool -x ./fwbin/${fwBin} -u ${driveID}   2>&1`
                                if [ $? -ne 0 ]; then
                                    # one and done
                                    fn_notifyGui "ERROR: Drive firmware update failed: on drive ${driveID}"
                                    fn_Log "    ERROR: Drive firmware update failed: on drive ${driveID}"
                                    if [  -z "${g_guiMode}" ]; then echo "";fi
                                    fn_notifyGui "${dtMsg}"
                                    fn_Log "    ${dtMsg}"
                                    if [  -z "${g_guiMode}" ]; then echo "";fi
                                    fn_notifyGui "*****The drive update has halted.  Please contact Technical Support before proceeding any further.*****"
                                    fn_Log "    *****The drive update has halted.  Please contact Technical Support before proceeding any further.*****"
                                    fn_Cleanup 1
                                else
                                    # verify that the FW version field changed
                                    newdriveFwRev=`fn_eqlDiskRevisionNumber $driveID`
                                    if [ "${newdriveFwRev}" = "${availableFwRev}" ] ; then
                                        fn_notifyGui "Drive firmware successfully updated : on drive ${driveID}"
                                        fn_Log "      Drive firmware successfully updated : on drive ${driveID}"
                                        if [  -z "${g_guiMode}" ]; then
                                            echo ""
                                        else
                                            # should this be a gui mode update then we need to calc/increment progress here
                                            TDLI=`fn_incrementProgressCalc "$TDLI"`
                                            g_currentProgress=$?
                                        fi
                                    else
                                        # one and done
                                        fn_notifyGui "ERROR:  Drive ${driveID} firmware update succeeded but diskview NOT displaying the new firmware revision number."
                                        fn_Log "    ERROR:  Drive ${driveID} firmware update succeeded but diskview NOT displaying the new firmware revision number."
                                        if [  -z "${g_guiMode}" ]; then echo "";fi
                                        fn_notifyGui "*****The drive update has halted.  Please contact Technical Support before proceeding any further.*****"
                                        fn_Log "    *****The drive update has halted.  Please contact Technical Support before proceeding any further.*****"
                                        fn_Cleanup 1
                                    fi
                                fn_GSBfix $driveID $driveModel 
                                fi
                            fi
                        else
                            fn_Log "    ERROR:  The chksum of the firmware update for this drive is invalid. skipping $driveID"
                        fi
                    else
                            fn_Log "    The current firmware on drive ID: ${driveID} is not a qualified Dell EqualLogic version"
                    fi
                fi
            else
                fn_Log "    SKIPPING DriveID ${driveID}  No firmware is available for  Model: ${driveModel}"
                fn_Log ""
                # update the progress count
                fn_progressMsg 1
            fi
        else
            # the drive is not reporting "Online... We should not hit this if the ecli hs is working correctly."
            fn_notifyGui "SKIPPING  Drive-ID $driveID state is ${driveStatus}"
            fn_Log "    SKIPPING  Drive-ID $driveID state is ${driveStatus}"
            fn_Log ""
            # update the progress count
            fn_progressMsg 1
        fi
    done
    return 0
}

GLIST_MAX=300           # Maximum # of glist entries acceptable
IDLE_MIN=10             # Minimum idle percentage

# set a trap right up front
trap fn_Cleanup 1 2 3 6 9

###############################################################################
###############################
#                             #
#   M A I N   P R O G R A M   #
#                             #
###############################
# main()

scriptName=`echo $0 | awk -v EXT=.sh -f ./alib/basename.awk`
# check if we are being run thru the gui... If so then set the one global we use.
if [ ! -z "$1" ]; then
    g_guiMode=`echo "$@" | awk 'BEGIN{ignorecase=1}/-gui/{print $1}'`

    if [ ! -z "${g_guiMode}" ]; then
        fn_Log "Gui Mode detected."
        shift
    fi
fi


####(NOTE:  we will call fn_Cleanup immediately from anywhere in this script should anything go wrong.)
fn_Log "Starting Disk Firmware update... V${VERSION}"


# get the service tag info right up front.
serviceTagData=`echo tv | ecli | awk 'BEGIN{IGNORECASE=1}/tag/'`
fn_Log "    Customer Service Tag Data" "T"
fn_Log "    #########################" "T"
fn_Log "    ${serviceTagData}"         "T"
fn_Log "    #########################" "T"

# if we detect M3 marvel or SiI (Silicon Image then quit.
notSupportedController=`diskview -I | awk 'BEGIN{IGNORECASE=1}/Marvel|SiI /'`
if [ ! -z "${notSupportedController}" ]; then
    fn_notifyGui  "Drive FW Upgrades are not supported on this platform"
    fn_Cleanup 3  "...  Drive FW Upgrades are not supported on this platform"
fi

silently=`disktool -s 0 >/dev/null 2>&1`

fn_Initialize
if [ $? -ne 0 ]; then
    fn_Log "Exiting with a status of 1"
    fn_Cleanup 1
fi

# secret swtich for sustaining only
# init our ignores
IGNORE_VERSION=0
while getopts ilrh o
do
    case "$o" in
        i)
            IGNORE_VERSION=1
            fn_Log "Disabling Version Checking"
            ;;
    esac
done

fn_progressMsg 4

CM_TYPE=`fn_GetCmStatus`
if [ $? -eq 1 ] ; then
    fn_notifyGui "Drive firmware update must be run on the active controller. "
    fn_LogEvent "Drive firmware update must be run on the active controller. "
    fn_Log "This procedure must be run on the active controller. "
    fn_Cleanup 1
fi

# if the user knew the secret handshake then they can skip the version check
# otherwise we enforce this set of maj,min,maint version requirements

if [ $IGNORE_VERSION -eq 0 ]; then
    curVersion=`fn_CheckVersionMinimum ${reqMajVer} ${reqMinVer} ${reqMaintVer}`
    if [ $? -ne 0 ]; then
        fn_notifyGui "The minimum supported version for this procedure is V${reqMajVer}.${reqMinVer}.${reqMaintVer}. You are running ${curVersion}"
        fn_Log "ERROR:  The minimum supported version for this procedure is V${reqMajVer}.${reqMinVer}.${reqMaintVer}. You are running ${curVersion}"
        fn_Cleanup 1
    fi
fi
# Stop any scanning, spin up any spun down spares
fn_Scanoff
if [ $? -ne 0 ]; then
    fn_notifyGui "Scanoff prep procedures failed"
    fn_Log "Exiting with a status of 1"
    fn_Cleanup 1
fi
fn_progressMsg 2

# Check overall health of Array
rmsg=`fn_GetRaidStatus`
rstat=$?
if [ $rstat -ne 0 ] && [  $rstat -ne 2 ] ; then
    fn_notifyGui  "${rmsg}"
    fn_Log "${rmsg}"
    fn_Log "Exiting with a status of 1"
    fn_Cleanup 1
fi

# ok RAID is good  now check ANY for bad/failed drives
badFailed=`diskview -I s |  awk 'BEGIN{IGNORECASE=1} /bad/'`
if [ ! -z "${badFailed}"  ]; then
    fn_notifyGui "Cannot proceed with  failed drives... ${badFailed}"
    fn_Log "Cannot proceed with  failed drives... ${badFailed}"
    fn_Log "Exiting with a status of 1"
    fn_Cleanup 1
fi
# Get the health status of the array.  Green means go and nothings else...
healthStatus=`fn_GetArrayHealthStatus`
if [ $? -ne 0 ]; then
    fn_notifyGui "Member is not in a GREEN health condition"
    fn_Log "$healthStatus"
    fn_Cleanup 1
fi

# ok RAID is good  now check for bad/failed drives
badFailed=`diskview -I s |  awk 'BEGIN{IGNORECASE=1} /bad/'`
if [ ! -z "${badFailed}" ]; then
    fn_notifyGui "Cannot proceed with  failed drives... ${badFailed}"
    fn_Log "Cannot proceed with  failed drives... ${badFailed}"
    fn_Log "Exiting with a status of 1"
    fn_Cleanup 1
fi

# Check to see if we still have drives to update
fn_Log  ""
fn_Log "    Identifying drives that qualify for firmware upgrades... Please wait."

# use the total of number of drives to be checked as a weight of the progress counter
#
fn_Updatecnt
if [ $? -ne 0 ] ; then
    fn_notifyGui  "Unable to get the list of upgradeable drives"
    fn_Log "Error: Unable to get the list of upgradeable drives"
    fn_Cleanup 1
fi

fn_Log "     ${updateableDriveCount} drive(s) in this array qualify for a firmware upgrade."
if [ ${updateableDriveCount} -eq 0 ]; then  # nothing to do!!!
    fn_Cleanup 0  "No drives in this array qualify for a firmware upgrade."
fi

### ASK VERIFICATION HERE
fn_Log ""
fn_Log ""
fn_Log "-----------------------------------------------"
fn_Log " The process will now update the ${updateableDriveCount} drives that qualify for a firmware upgrade."
fn_Log ""
fn_Log " If you proceed, please do not power off or restart the array, or remove"
fn_Log " any drives until the update process completes."
fn_Log "-----------------------------------------------"
# if we are  not in gui mode then ask the user if they wish to proceed
if [  -z "$g_guiMode" ]; then
    fn_AskYesOrNo "Do you want to continue at this time "
    if [ $? -ne 0 ]; then
        fn_Cleanup 2
    fi
fi

#
#Here we go.
# Update the drives
fn_UpdateDriveFirmware ${updateableDriveCount}
if [ $? -ne 0   ]; then
    fn_notifyGui "Please contact support to review the Drive firmware update Log"
    fn_Cleanup 1
fi

# fall thru

    fn_notifyGui "Drive Firmware Update completed successfully."
fn_Cleanup 0
