#!/bin/sh
#
# @(#) Script to consolidate all the common functions used by fw update scripts
#
# Copyright (c) 2010-2017 by Dell Inc.
#
# All rights reserved.  This software may not be copied, disclosed,
# transferred, or used except in accordance with a license granted
# by Dell Inc.  This software embodies proprietary information
# and trade secrets of Dell Inc.
#
#

# ------------------------------------------
#   Function:       fn_Log(message, [quietlyLog])
#
#   Description:
#      Log/display a timestamped message to a file with the basename of the executing script and a .log
#      extension.
#
#   Inputs:
#       _message_   A message to log
#
#       _quietlyLog_  Optional arg that when non null prevents message from going to stdout
#              the message will only go to the log. Useful for very verbose outputs
#
#  Outputs:
#       A timestamped user supplied message is output to both stdout and a log
#       file with a basename respective of the script name.
#
#   Returns:
#       null
#
fn_Log ()
{
    message="$1"
    quietlyLog="$2"
    curDT=`date`

    scriptName=`echo $0 | awk -v EXT=.sh -f ./alib/basename.awk`
    if [  -z "${g_guiMode}" ]; then
        if [  -z "$quietlyLog" ]; then
            echo "  $message"
        fi
    fi
    if [ -d "/var/update" ]; then
        echo "$curDT    $message" >> /var/update/${scriptName}.log
    fi
}
# ------------------------------------------
#       Function:       progress_msg()
#
#   Description:
#               Function to show the indicated level of progress
#               when in GUI mode
#
#       Inputs:
#               _addToProgress_   amount(number) of progress to increment the current progress value
#
#
#       Outputs:
#           echo of overallProgress to stderr for the -gui progress tracking
#
#       Returns:
#           none
#
fn_progressMsg()
{
    addToProgress="$1"
    g_currentProgress=`echo ${g_currentProgress}  ${addToProgress} | awk '{print $1 + $2}'`
    # never show 99 or greater
    if [ ${g_currentProgress} -ge 99 ]; then
        g_currentProgress=98
    fi
    if [  ! -z "$g_guiMode" ]; then
        echo "Progress: ${g_currentProgress}" >&2
        fn_Log "Current Progress: ${g_currentProgress}" "T"
    fi

    return $g_currentProgress
}
# ------------------------------------------
#   Function:       fn_Initialize()
#
#   Description:
#      Perform common initialization tasks.
#      Tasks are:
#          - Initialize the mib infrastructure.
#
#   Inputs:
#       none
#
#   Outputs:
#       none
#
#   Returns:
#       0 (success)
#
#       1 (error)
#
fn_Initialize()
{
    curdir=$PWD;
    fn_Log "...Initializing support libraries..."
    if [ ! -d /mgtdb/update/mibwalk ]; then
        mkdir -p /mgtdb/update/mibwalk
        cd /mgtdb/update/mibwalk
        gzip -dc /usr/share/snmp/snmp.tgz 2>/dev/null | tar xf -
    fi

    cd ${curdir}
    if [ ! -d /mgtdb/update/mibwalk ]; then
        fn_Log "ERROR: Unable to initialize support libraries"
        return 1
    fi

    return 0
}
# ------------------------------------------
#   Function:       fn_AskYesOrNo()
#
#   Description:
#      Prompts the user to answer a Y or N question
#
#   Inputs:
#       The yes or no question(string" to ask the user
#
#   Outputs:
#       nothing is output if the question is answered.
#       A warning is output if the user can't get the Y/N answered after $tries.
#
#   Returns:
#       0   a Y  response was received
#
#       1   a N  response was received or the user didn't respond appropriately after $tries
#
fn_AskYesOrNo()
{
    riddleMeThis="${1}"
    tries=1
    ANSWER=""

    while [  "${ANSWER}" != "Y" -o "${ANSWER}" != "N" ]
    do
        fn_Log  "    ${riddleMeThis} (Y/N)? " "T"
        echo -n "    ${riddleMeThis} (Y/N)? "
        read ANSWER
        fn_Log "The user responded with ${ANSWER} on try ${tries}"  "T"
        case "${ANSWER}" in
            "n" | "N" | "no"  | "NO"  | "No"  | "nO" ) return 1;;
            "y" | "Y" | "yes" | "YES" | "Yes" | "YEs" | "yEs" | "yES" )  return 0;;
        esac
        tries=`echo $tries | awk '{print $0 + 1}'`
        if [ $tries -gt 3 ]; then
            fn_Log "    WARNING: Max tries exceeded in answering the Y/N question by the user."
            fn_Log "    Defaulting to a N response."
            return 1
        fi
        ANSWER=""
    done
}

# ------------------------------------------
#   Function:       fn_GetSmart(driveID diskviewOption)
#
#   Description:
#      Log diskview output of 3 possible option -B -T  and -F
#   Inputs:
#       _driveID_    the drive id to get the diskview output for
#       _diskviewOption_  One of 3 possible options(see description)
#
#  Outputs:
#       The output of diskview OR a message indicating that the data was unavailable.
#
#   Returns:
#       0 Requested Output was successfuly obtained
#       1 unable to obtain requested output
#

fn_GetSmart()
{
    driveID=$1
    diskviewOption=$2
    # optional arg

    smartDeviceData=""
    tries=0
    maxTries=6
    ######
    driveStatus=`diskview -i $driveID | awk '/State:/ {print $7 $8}' 2>&1`
    if  [ "$driveStatus" = "online" ] || [ "$driveStatus" = "Online" ]; then
        # this is our marker var.
        RSC=""
        while [ -z "${smartDeviceData}" ]
        do
            smartDeviceData=`diskview -${diskviewOption} ${driveID}  2>&1`
            pdioctlHit=`echo "${smartDeviceData}" | awk 'BEGIN{IGNORECASE=1} /ioctl/'`
            if [ ! -z "${pdioctlHit}" ];then
                tries=`echo $tries | awk '{print $0 + 1}' `
                smartDeviceData=""
                sleep 2   # deep cleansing breaths before we retry...
            else
                if [ "${diskviewOption}" = "F" ]; then
                    RSC=`echo "${smartDeviceData}" | awk 'BEGIN{IGNORECASE=1} /EXCEEDED/ {print $0} ' 2>&1`
               fi
            fi
            # well do we try again?
            if [ $tries -gt $maxTries  ]; then
                fn_Log "###################################################" "T"
                fn_Log "    Drive ${driveID}  Smart Device Data unavailable" "T"
                fn_Log "###################################################" "T"
                return 1
            fi

        done
        # We got it, so choose the appropriate output
        if [ "${diskviewOption}" = "F" ]; then
            fn_Log "    ${RSC}" "T"
        fi

        if [ "${diskviewOption}" = "B"  -o "${diskviewOption}" = "T" ]; then
            fn_Log "###################################################" "T"
            fn_Log " Drive ${driveID}"           "T"
            fn_Log "####################"        "T"
            fn_Log "${smartDeviceData}"          "T"
        fi
        fn_Log "----------------------------------------------------------------------" "T"
        return 0
    else
        fn_Log "################################################################"       "T"
        fn_Log "    Drive ${driveID} is not online Smart Device Data unavailable"       "T"
        fn_Log "################################################################"       "T"
        fn_Log "----------------------------------------------------------------------" "T"
        return 1
    fi
    # should never hit this
    return 0
}


# ------------------------------------------
#   Function:       fn_Cleanup([exitStatus])
#
#   Description:
#      Function to terminate the script gracefully
#
#   Inputs:
#       _exitStatus_  optional integer indicating the exit status
#       _specialMessage_  optional string to append to the emhome subject
#
#   Outputs:
#   A series of messages indicating that we are cleaning up and exiting.
#   Any led's we turned on are turned off and enable TA scan thread
#
#
#   Exit:
#       0  (success) script terminated normally
#
#       1  (error)  script terminated deliberately or by signal(contact technical support).
#
#       2  (quit) script terminated when user decided not to proceed.(special case)
#
#       3  (attention) script requires user to contact technical support
#
fn_Cleanup()
{
    # pull in sendemail
    . /sbin/sendEmail.sh
    if [ $# -eq 0 ]; then
        exitStatus=1
        fn_Log "    Caught Signal ... cleaning up."
        fn_Scanon
    else
        exitStatus=$1
    fi
    if [  ! -z "$g_guiMode" ]; then echo "Progress: 100" >&2; fi
    specialMessage="$2"
    # get the script name
    scriptName=`echo $0 | awk -v EXT=.sh -f ./alib/basename.awk`
    #
    # get the controller type and then filter/normalize the string
    tries=0
    cType=""
    while [  -z "${cType}" ]
    do
        cType=`echo cm | ecli | awk -F: 'BEGIN{IGNORECASE=1}/Type:/{type=$2}END{print type}' | sed 's/ //g'`
        tries=`echo $tries | awk '{print $0 + 1}'`
        if [ $tries -gt 3 ]; then
            cType="unknown"
        fi
    done

    fn_Log "    Controller type detected: ${cType}" "T"

    fn_Log "    Logging post run disk information"
    fn_Log "=============== diskview -I s =====================" "T"
    dbis=`diskview -I s `
    fn_Log "${dbis}" "T"
    fn_Log "==================================================="
    fn_Log "    Entering cleanup phase... Please Wait..."

    # re-enable TA scan thread
    fn_Scanon

    # if we persist and find an SMTP server then email the log file
    mailserver=`fn_GetEmailServer`

    # over-ride the subject if the user quit.(53470)
            # the  normal exit code 0  subject
        # Special case over-ride the subject if we skipped a drive on type 5 controller
    if [  $exitStatus -eq 3 ];then
        subject="ATTENTION on ${cType} Controller ${specialMessage}"
        fn_Log "    ATTENTION on ${cType} Controller ${specialMessage}"
        fn_Log "    Please contact Technical Support..."
    elif [  $exitStatus -eq 2 ];then
        subject="User chose to QUIT  before updating on ${cType} Controller ${specialMessage}"
        if [ ! -z "$mailserver"   ];then
            fn_Log "    ${subject} ... No email will be processed/sent"
        fi
    elif [ $exitStatus -eq 1 ]; then
        subject="FAILED on ${cType} Controller ${specialMessage}"
        fn_Log "    The update failed - Please contact Technical Support"
    else
        subject="SUCCESS on ${cType} Controller ${specialMessage}"
        fn_Log "    The update was successful. ${specialMessage}"
    fi

    # round up all the email addresses we know about.
    eventEmailAddr=`fn_GetMib "eqlGroupAlertEmailAddress"`
    emHomeEmailAddr=`fn_GetMib "eqlGroupEmailSupportContact"`
    okToPhoneHome=1
    # First check if we have SMTP setup or not...
    if [ ! -z "$mailserver"  -a $exitStatus -ne 2 ]; then
        # are we in gui mode?
        if [  -z "$g_guiMode" ]; then
            # not in gui mode so ask for permission to sent email home
            fn_AskYesOrNo "Would you like to email the results to Dell  "
            okToPhoneHome=$?
        else   # we are in gui mode and will send mail only if emHome is set.
            if [ ! -z "${emHomeEmailAddr}" ]; then
                okToPhoneHome=0
            fi
        fi
        #
        if [ $okToPhoneHome -eq 0 ]; then
            # this first one we know for sure.
            EMailAddr="eql_hdd_update_notify@dell.com"
            # if set send email to the eqlGroupAlertEmailAddress
            if [ ! -z "${eventEmailAddr}" ]; then
                EMailAddr="${EMailAddr},${eventEmailAddr}";
            fi
            # if set send email to eqlGroupEmailSupportContact(aka emHome)
            if [ ! -z "${emHomeEmailAddr}" ]; then
                EMailAddr="${EMailAddr},${emHomeEmailAddr}";
            fi

            #Log the status here so it gets included in the email.
            fn_Log "    Done cleanup ... quitting with an exit status of $exitStatus"
            # send the email
            mailMessage=`SendEMail "${scriptName} ${subject}"  "/var/update/${scriptName}.log"`
            if [ $? -ne 0 ]; then
                fn_Log "    Error sending e-mail." "T"
                fn_Log "    Error message is ${mailMessage}" "T"
            fi
        else
            if [ $okToPhoneHome -eq 1 ]; then
                if [ -z "$emHomeEmailAddr" ]; then
                    fn_Log "    NOTE: EMHOME is not set "  "T"
                else
                    fn_Log "    NOTE: EMHOME is set "  "T"
                fi
            else
                fn_Log "    User answered NO to emailing results to Dell"  "T"
            fi
            fn_Log "    Done cleanup ... quitting with an exit status of $exitStatus"
        fi
        echo ""
    else
        fn_Log "    No mailserver/smtp server address was found." "T"
        fn_Log "    Done cleanup ... quitting with an exit status of $exitStatus"
    fi

    # clean up the mibwalk area
    rm -rf /mgtdb/update/mibwalk

    exit $exitStatus
}

# ------------------------------------------
#   Function:       fn_IsNumeric(value)
#
#   Description:
#      Function to determine if a value is numeric
#
#   Inputs:
#       _value_ argument to test for integer-ness
#
#   Outputs:
#       none
#
#   Returns:
#       0  (success) input value is numeric
#
#       1  (error) input value is NOT numeric
#
fn_IsNumeric()
{
    value="$1"

    # check the special case where we are talking about drive ZERO
    ZBUCKET=`echo "$value" | awk '{ if ($0 == 0){print "ZERO"}} '`
    if [ ! -z "$ZBUCKET"  ]; then
        return 0
    fi
    # check for non-numeric
    isNotNumber=`echo $value | awk '{ print ($0==$0+0) } '`
    if [ "$isNotNumber" -eq 0 ]; then
        return 1
    fi
    # check for negative number
    ret=`echo $value | awk '{ print log($0);}' 2>&1`
    isnegative=`echo $ret | awk '/-inf/ {print "negative number detected"}'`
    if [ ! -z "$isnegative" ]; then
        return 1
    fi

    # looks goods
    return 0
}
# ------------------------------------------
#   Function:       fn_GetDriveCount()
#
#   Description:
#      Function to count the drives seen by diskview
#
#   Inputs:
#       none
#
#   Outputs:
#       drivecount for that array is output
#
#   Returns:
#       0 (success) Drive count was detected and recieved
#
#       1 (error) Drive count was NOT detected and recieved
#
fn_GetDriveCount()
{
    dCount=`diskview -I s | awk '/^Disk/ { count++ } END { print count }'`
    if [ -z "$dCount" ]; then
        return 1
    fi
    echo "$dCount"
    return 0
}

# ------------------------------------------
#   Function:       fn_GetEmailServer()
#
#   Description:
#          Routine to return the Email Server Ip Address
#
#   Inputs:
#         none
#
#   Outputs:
#           the email server IP address
#
#   Returns:
#       0  (success) the mail server ip address was found
#
#       1  (error) the mail server ip address was NOT found
#
fn_GetEmailServer()
{
    # If the running version  is less than Macallan(V4.0.0), do
    # not use the eqllogger command to get the mailserver. Do it the
    # old way
    curVersion=`fn_CheckVersionMinimum 4 0 0`
    if [ $? -eq 0 ] ; then
        mailserver=`fn_GetMib "eqlGroupSMTPServerIpAddress"`
    else
        mailserver=`/sbin/eqllogger -M | awk '/SmtpServerAddress/  { str=$2; gsub(/\[/, "", str); gsub(/\]/, "", str)} END {print str}'`
    fi
    if [ -z "$mailserver" ]; then
        return 1
    else
        echo  "$mailserver"
        return 0
    fi

}

# ------------------------------------------
#   Function:  fn_VerifyChecksums(descriptorFile)
#
#   Description:
#       This routine verifies that the information contained in a drive models descriptor file
#       is present and validated
#
#   Inputs:
#       _descriptorFile_   descriptor file containing cksum , size and binary file information
#
#  Outputs:
#      none
#
#   Returns:
#       0 (success) cksum was ok
#
#       1 (error) cksum was NOT ok
#
fn_VerifyChecksums()
{
    # Make sure that the firmware bin files chksum correctly
    #
    descriptorFile="$1"

    if [ ! -s "${descriptorFile}" ]; then
        fn_Log "    Descriptor file $descriptor not found" "T"
        return 1
    fi
    #
    # define our field of interest(for readability mostly)
    ckSumField=1
    sizeField=2
    binFileField=3
    #
    expBinCksum=`cat $descriptorFile | awk -v WHICH_FIELD=$ckSumField   -f ./alib/readFieldInCommentedFile.awk`
    expBinSize=`cat $descriptorFile  | awk -v WHICH_FIELD=$sizeField    -f ./alib/readFieldInCommentedFile.awk`
    if [ -z "$expBinCksum" ]  || [ -z "$expBinSize" ]; then
        fn_Log "    ERROR: Unable to read the expected checksum."
        return 1
    fi
    #
    binFile=`cat $descriptorFile  | awk -v WHICH_FIELD=$binFileField -f ./alib/readFieldInCommentedFile.awk `

    if [ -s "./fwbin/${binFile}" ]; then
        cksumOut=`cksum ./fwbin/${binFile}`
        if [ -z "$cksumOut" ]; then
            fn_Log "    ERROR: Null return from chksum check" "T"
            return 1
        fi
        # go get the cksum now and see if it matches what is expected
        rcvBinCksum=`echo $cksumOut | awk -v WHICH_FIELD=$ckSumField   -f ./alib/readFieldInCommentedFile.awk`
        rcvBinSize=`echo $cksumOut  | awk -v WHICH_FIELD=$sizeField    -f ./alib/readFieldInCommentedFile.awk`
        match1=`echo ${expBinCksum} ${rcvBinCksum}  | awk '{ if ( $1 != $2){print 1} else {print 0} }'`
        match2=`echo ${expBinSize} ${rcvBinSize}  | awk '{ if ( $1 != $2){print 1} else {print 0} }'`
        if [ "${match1}" -ne "0" ] || [ "${match2}" -ne "0" ]; then
            fn_Log "    ERROR: Checksum failed"
            return 1
        fi
    else
        # No cksums file, or it is null -- clearly bad.
        fn_Log "    firmware file in ./fwbin/${binFile} missing or null" "T"
        return 1
    fi
    return 0
}
# ------------------------------------------
#   Function:       fn_GetDriveState(driveToCheck)
#
#   Description:
#      Gets the state of the specified drive id
#
#   Inputs:
#       _driveToCheck_ The numerical drive ID to check
#
#   Outputs/:
#       If drive id  is not Online then current drive status is output
#       If the drive id is online then no messages are output
#
#   Returns:
#       0  (success) drive is online
#
#       1  (error) drive is NOT online
#
#
fn_GetDriveState()
{
    driveToCheck=$1
    status=`diskview -I s | awk "/ ${driveToCheck} /" | awk '/nline/{print $0}'`
    if [ -z "$status" ]; then
        fn_Log "    ERROR: Drive $driveToCheck is NOT Online"
        fn_Log "${status}"
        return 1
    fi
    return 0
}


# ------------------------------------------
#   Function:	fn_Scanoff()
#
#   Description:
#       Turn passive scan off
#
#   Inputs:
#	none
#
#   Outputs:
#       none
#
#   Returns:
#     0 (success) Scanning turned off
#
#     1 (error) Scanning NOT turned off
#
fn_Scanoff()
{

    fn_Log "    Disable scanning and preemptive drive removal" "T"
    # disable preemptive drive removal
    disktool -Z 0
    SCAN_DISABLED=`diskview -G | awk '/DISABLED/ {print "DISABLED"}'`
    if [ -z "${SCAN_DISABLED}" ] ; then
        # disable Background scan thread
        disktool -c 0 >/dev/null 2>&1
        # disable TA scan thread
        disktool -J 0 >/dev/null 2>&1
        sleep 5
    fi

    curVersion=`fn_CheckVersionMinimum 5 0 0`
    if [ $? -eq 0 ] ; then
        # disable spindown of spares
        quietly=`disktool -s 0 >/dev/null 2>&1`
    fi
    scanStatus=`diskview -I s | awk 'BEGIN{ IGNORECASE=1 }; /scanning/'`
    if [ -z $scanStatus ]; then
        return 0
    else
        fn_Log "    ERROR: Unable to turn passivescanning off"
        return 1
    fi
}

# ------------------------------------------
#   Function:	fn_Scanon()
#
#   Description:
#       Turn passive scan on
#
#   Inputs:
#	none
#
#   Outputs:
#	none
#
#   Returns:
#     0 (success) Scanning turned on
#
#     1 (error) Scanning NOT turned on
#
fn_Scanon()
{
    fn_Log "     Enable scanning and preemptive drive removal" "T"
    SCAN_DISABLED=`diskview -G | awk '/DISABLED/ {print "DISABLED"}'`
    if [ ! -z "${SCAN_DISABLED}" ] ; then

        # Enable TA scan thread
        disktool -J 1 >/dev/null 2>&1
    fi
    # Enable preemptive drive removal
    disktool -Z 1
    curVersion=`fn_CheckVersionMinimum 5 0 0`
    if [ $? -eq 0 ] ; then
        quietly=`disktool -s 1 >/dev/null 2>&1`
    fi

    return 0
}

# ------------------------------------------
#   Function:   fn_GetArrayHealthStatus()
#
#   Description:
#      Check the health status of the array via 'ecli'
#
#   Inputs:
#       none
#
#   Outputs:
#       When the health status of the array is anything but
#           either
#       0x0000000000000(ie GREEN)
#            OR
#       0x0000000000000004): (ie YELLOW Conditions: CONTROL_MODULE_REMOVED_CONDITION)
#       a warning message is output
#            OR
#      (0x0000000004000000): YELLOW Conditions: NO_ETHERNET_FLOW_CONTROL
#
#   Returns:
#       0  (success) array is healthy(green)
#
#       1  (error) array is unhealthy(non-green)
#
fn_GetArrayHealthStatus()
{
    fn_Log "    Checking health status of Array"
    # check the health status of the array
    health=`echo hs | ecli`
    isHealthy=`echo $health | awk '/0x0000000000000000|0x0000000000000004|0x0000000004000000/'`
    if [  -z "$isHealthy" ]; then
        fn_Log "    Warning: array is not in a GREEN health condition"
        fn_Log "    $health"
        return 1
    fi

    return 0
}
# ------------------------------------------
#   Function:  fn_CheckUsageOfSpecifiedDrive(userSpecifiedDrive)
#
#   Description:
#      Check the usage of a used specified drive
#
#   Inputs:
#       _userSpecifiedDrive_ The drive to inquire usage about.
#
#   Outputs:
#      Message indicating if the user specified drive is a SPARE drive or a RAIDSET drive.
#
#   Returns:
#           0  (success) if the user specified drive is a spare drive
#
#           1  (error) if the user specified drive is a raidset drive
#
fn_CheckUsageOfSpecifiedDrive()
{
    userSpecifiedDrive="$1"
    spareDrives=`raidtool -Z | awk '/hot spare/' | sed 's/=/ /' | awk '{print $3}'`
    for s in $spareDrives
    do
        if [ $s -eq $userSpecifiedDrive ]; then
            echo "SPARE"
            return 0
        fi
    done
    raidsetDrives=`raidtool -Z | awk '/active/' | sed 's/=/ /' | awk '{print $3}'`
    for  d in $raidsetDrives
    do
        if [ $d -eq $userSpecifiedDrive ]; then
            echo "RAIDSET"
            return 1
        fi
    done
}

# ------------------------------------------
#   Function:       fn_GetDriveSize(driveToCheck )
#
#   Description:
#      Get the size of the specified drive id
#
#   Inputs:
#       _driveToCheck_ The numerical drive ID to check
#
#   Outputs:
#       A message indicating the size (in blocks) of the requested drive id.
#
#   Returns:
#       0  (success) successfully obtained size
#
#       1  (error) failed to obtain size
#
#
fn_GetDriveSize()
{
    driveToCheck=$1
    # The next line accommodates shrunk drives as well as normal ones.
    size=`diskview -i $driveToCheck | awk '/blocks/{ print $0 }' | awk 'BEGIN { FS="=" }; { print $2 }' |awk '/blocks/ { print $1 }'`
    echo $size
    if [ "${size}" \< "0" ]; then
        return 1
    fi

    return 0
}
# ------------------------------------------
#   Function:   fn_SwapDriveOut(driveToSwap)
#
#   Description:
#      Method to guide the user to the correct drive to swap out.
#
#   Inputs:
#       _driveToSwap_  The numerical drive ID to check
#
#   Outputs:
#       A timestamped user supplied message is output to both stdout and a log
#       file with a basename respective of the script name.
#   Returns:
#       0  (success) successful when the drive is swapped successfully
#
#       1  (error) failure when the drive swap fails`
#
fn_SwapDriveOut ()
{
    driveToSwap="$1"
    fn_Log ""
    beforeModelLine=`diskview -i ${driveToSwap} | awk '/Model/'`
    beforeSize=`fn_GetDriveSize $driveToSwap`
    fn_Log "NOTE: WE WILL TEMPORARILY ILLUMINATE THE FAULT LIGHT ON DRIVE $driveToSwap  TO MAKE IT EASIER TO FIND"

    quietly=`echo fon $driveToSwap | ecli`
    echo ""
    fn_AskYesOrNo "Enter (Y) to continue AFTER replacing drive $driveToSwap or (N) to quit now: "
    if [ $? -ne 0 ]; then
        quietly=`echo foff $driveToSwap | ecli`
        fn_Cleanup 2
    fi
    echo ""
    fn_Log "    Checking Drive $driveToSwap readiness .."
    sleep 60
    # Drive should be online now
    fn_GetDriveState ${driveToSwap}
    if [ $? -ne 0 ]; then
        currentState=`diskview -i ${driveToSwap} s`
        fn_Log "    Drive:  ${driveToSwap}  is reporting a status of "
        fn_Log "    ${currentState}"
        fn_Log "Exiting with a status of 1"
        return 1
    fi
    afterModelLine=`diskview -i ${driveToSwap} | awk '/Model/'`
    if [ "${beforeModelLine}" =  "${afterModelLine}" ]; then
        fn_Log "    Warning: The drive you just replaced is the same drive you were just asked to remove earlier"
        fn_Log "    Exiting with a status of 1"
        return 1
    fi
    afterSize=`fn_GetDriveSize $driveToSwap`

    sizeError=`echo $beforeSize  $afterSize | awk '{ if ($1 > $2) {print "1"} else {print "0"}}'`
    if [ ${sizeError} -eq 1 ]; then
        fn_Log "    Warning: The drive you just replaced is smaller than the original  drive"
        fn_Log "Exiting with a status of 1"
        return 1
    fi

   return 0
}



# ------------------------------------------
#   Function:       fn_GetMib(mibOid)
#
#   Description:
#       Function to get a MIB variable
#
#   Inputs:
#     _mibOid_    oid value of the MIB variable to fetch.
#
#   Outputs:
#     output the value for the requested oid.
#
#   Returns:
#     0 (success) if the variable was obtained
#
#     1 (error) and MIBVAL is null if the variable is not present in this system.
#
fn_GetMib()
{
    mibOid="$1"
    curdir=$PWD;

    if [ ! -d /mgtdb/update/mibwalk ]; then
        mkdir -p /mgtdb/update/mibwalk
        cd /mgtdb/update/mibwalk
        gzip -dc /usr/share/snmp/snmp.tgz 2>/dev/null | tar xf -
    fi

    cd /mgtdb/update/mibwalk
    export LD_LIBRARY_PATH=/usr/lib:/mgtdb/update/mibwalk/lib

    MIBVAL=`/mgtdb/update/mibwalk/bin/snmpwalk -c public -Oqv -M mibs -m all -t 33 localhost ${mibOid} 2>/dev/null`
    retVal=$?
    if [ $? -ne 0 ]; then
        MIBVAL=
    fi
    echo "$MIBVAL"
    cd ${curdir}
    return $retVal
}


# ------------------------------------------
#   Function:       fn_CheckVersionMinimum(reqMajVer, reqMinVer, reqMaintVer)
#
#   Description:
#       Function to get a check the current version against a minumum version
#
#   Inputs:
#      _reqMajVer_   is the minimum major version number
#
#      _reqMinVer_  is the minimum minor version number
#
#      _reqMaintVer_ is the minimum maintenance number
#
#   Outputs:
#      outputs the current version string
#
#   Returns:
#      0 (success) the minimum version criteria is met
#
#      1 (error) the minimum version criteria is NOT met
#
fn_CheckVersionMinimum()
{
    reqMajVer=$1
    reqMinVer=$2
    reqMaintVer=$3

    # use the ecli cm command to get the sw rev.
    curVersion=`echo cm | ecli | awk '/SW Revision/{ sub(/V/, "", $6); ver=$6} END{ print ver}'`
    curMajVer=`echo $curVersion | awk -F.   '{print $1}'`
    curMinVer=`echo $curVersion | awk -F.   '{print $2}'`
    curMaintVer=`echo $curVersion | awk -F. '{print $3}'`

    # if for any reason any of these are null then move to plan b
    if [ -z "$curMajVer" ] || [ -z "$curMinVer" ] || [ -z "$curMaintVer" ]; then
        running_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /build.c)
        if [ -r /var/patch/build.c ] ; then
            running_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /var/patch/build.c)
        fi
        # strip off leading "storage array firmware " and trailing build ID.
        VERSION="${running_ver##* V}"
        VERSION="V${VERSION%% *}"
        curMajVer=`echo $VERSION | awk 'BEGIN {FS="."}; { sub ("V", "", $1 ); print $1 }'`
        curMinVer=`echo $VERSION | awk 'BEGIN {FS="."}; { print $2 }'`
        curMaintVer=`echo $VERSION | awk 'BEGIN {FS="."}; { print $3 }'`
    fi
    echo "V${curMajVer}.${curMinVer}.${curMaintVer}"
    # pad the leading zero
    curMajVer=`echo "${curMajVer}"     | awk '{  printf "%02d\n", $0}'`
    curMinVer=`echo "${curMinVer}"     | awk '{  printf "%02d\n", $0}'`
    curMaintVer=`echo "${curMaintVer}" | awk '{  printf "%02d\n", $0}'`
    #
    reqMajVer=`echo "${reqMajVer}"     | awk '{  printf "%02d\n", $0}'`
    reqMinVer=`echo "${reqMinVer}"     | awk '{  printf "%02d\n", $0}'`
    reqMaintVer=`echo "${reqMaintVer}" | awk '{  printf "%02d\n", $0}'`


    if [ "${curMajVer}.${curMinVer}.${curMaintVer}" \< "${reqMajVer}.${reqMinVer}.${reqMaintVer}" ]; then
        return 1
    fi
    return 0
}

# ------------------------------------------
#   Function:       fn_CheckIfOkToProceed
#
#   Description:
#      Convenience method to check overall health of member
#
#   Inputs:
#       None
#
#   Outputs:
#       Log file messages.
#
#   Returns:
#       0  (success) ok to proceed
#
#       1  (error) NOT ok to proceed
#

fn_CheckIfOkToProceed()
{

    fn_GetArrayHealthStatus
    if [ $? -ne 0 ]; then
        return 1
    fi

    fn_GetRaidStatus
    if [ $? -ne 0 ]; then
        return 1
    fi

    return 0
}

# ------------------------------------------
#   Function:       fn_GetCmStatus()
#
#   Description:
#      Convenience method to get B2B status and parse it and determine if
#      this is the active or passive cm
#
#   Inputs:
#       none
#
#  Outputs:
#       Echo the status of either "active" or "passive"
#
#   Returns:
#       0 (success) cm is active
#
#       1 (error) cm is passive
#
fn_GetCmStatus()
{
    #
    # Get B2B status and parse it.
    #
    # cord -b: returns passive => 0, active => 1, error => 2.
    B2BStat=$(cord -b)

    case "${B2BStat}" in
    *Active?CM*)
        echo "active"
        return 0
        ;;
    *)
        echo "passive"
        return 1
        ;;
    esac

}
