#!/bin/sh
#
# @(#) Script to define a set of constants use to access specific disk slot entries in the eqlDiskTable MIB
#
# Copyright (c) 2010-2017 by Dell Inc.
#
# All rights reserved.  This software may not be copied, disclosed,
# transferred, or used except in accordance with a license granted
# by Dell Inc.  This software embodies proprietary information
# and trade secrets of Dell Inc.
#
#

. ./lib/commonLib.sh

###########################################
## EqlDisk helper functions
###########################################

# ------------------------------------------
#   Function:   fn_FilterModelLine(modelLine)
#
#   Description:
#      Filter the vendor names out of the drive model string
#
#   Inputs:
#       _modelLine_  Model line string from diskview -i #
#
#   Outputs:
#       echo a filtered model line
#
#   Returns:
#       0  (success) Model line was received and filtered
#
#       1  (error)  Model line was null or not received
#
fn_FilterModelLine()
{
    modelLine="$1"
    if [ -z "$modelLine" ]; then
        return 1
    else
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/SEAGATE/, "", $0) } END{ print $0 }'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/SAMSUNG/, "", $0) } END{ print $0 }'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/WDC/,     "", $0) } END{ print $0 }'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/HITACHI/, "", $0) } END{ print $0 }'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/,/,       "", $0) } END{ print $0 }'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/=/,       " ", $0)} END{print $0}'`
        modelLine=`echo $modelLine | awk 'BEGIN{IGNORECASE=1} { gsub(/FWrev=/, "FWrev ", $0) } END{ print $0 }'`
    fi
    echo $modelLine
    return 0
}
############################################
### EqlDisk Getter functions
############################################

# ------------------------------------------
#
#   Function:   fn_eqlDiskType(diskID)
#
#   Description:
#      get the disk Type for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the type information about
#
#   Outputs:
#       echo the diskType
#
#   Returns:
#
#       0  (success) Disk type was recognized and displayed upper case
#
#       1  (error) Disk type was not recognized.
#
fn_eqlDiskType()
{
    diskID="$1"
    disktype=`diskview -i $diskID | awk 'BEGIN{IGNORECASE=1} /size/' |
                                     awk 'BEGIN{IGNORECASE=1}
                                         /SAS/  {print "SAS"};
                                         /SATA/ {print "SATA"};
                                         /SED/  {print "SED"};
                                         /SSD/  {print "SSD"};'`

    if [ -z "$disktype" ]; then
        return 1
    else
        echo $disktype | awk -f ./alib/toupper.awk
        return 0
    fi

}
# ------------------------------------------
#   Function:   fn_eqlDiskModelNumber(diskID)
#
#   Description:
#      get the disk Model for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the model information about
#
#   Outputs:
#       echo the disk Model number
#
#   Returns:
#
#       0  (success) Disk Model number was detected, filtered and displayed
#
#       1  (error) Disk Model number was not detected or received.
#
fn_eqlDiskModelNumber()
{
    diskID="$1"
    # get the entire model number line.
    driveModelLine=` diskview -i $diskID | awk '/Model:/ {print $0}'`
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        #filter out any vendor names in the model line
        driveModelLine=`fn_FilterModelLine "${driveModelLine}"`
    fi
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        # Now get the actual model from the filtered line
        driveModel=`echo $driveModelLine  | awk '/Model:/ {print $2}'`
    fi
    # either we have it now or not.
    if [ -z "$driveModel" ]; then
        return 1
    else
        echo $driveModel
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskRevisionNumber(diskID)
#
#   Description:
#      get the disk FW Rev for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the rev information about
#
#   Outputs:
#       echo the disk revision number(aka firmware)
#
#   Returns:
#
#       0  (success) Disk firmware rev was detected and received.
#
#       1  (error) Disk firmware rev was NOT detected or received.
#
fn_eqlDiskRevisionNumber()
{
    diskID="$1"
    # get the entire model number line.
    driveModelLine=` diskview -i $diskID | awk '/Model:/ {print $0}'`
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        #filter out any vendor names in the model line
        driveModelLine=`fn_FilterModelLine "${driveModelLine}"`
    fi
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        driveFwRev=` echo $driveModelLine | awk '/Model/ {fwRev=$6; gsub(/)/, "", fwRev ); print fwRev }'`
    fi
    if [ -z "$driveFwRev" ]; then
        return 1
    else
        echo $driveFwRev
        return 0
    fi
}
# ------------------------------------------
#   Function:   fn_eqlDiskSerialNumber(diskID)
#
#   Description:
#      get the disk Serial Number for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the Serial Number information about
#
#   Outputs:
#       echo the disk serial number
#
#   Returns:
#
#       0  (success) Disk serial number was detected and received.
#
#       1  (error) Disk serial number was NOT detected or  received.
#
fn_eqlDiskSerialNumber()
{
    diskID="$1"
    # get the entire model number line.
    driveModelLine=` diskview -i $diskID | awk '/Model:/ {print $0}'`
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        #filter out any vendor names in the model line
        driveModelLine=`fn_FilterModelLine "${driveModelLine}"`
    fi
    if [ -z "$driveModelLine" ]; then
        return 1
    else
        # Now get the actual model from the filtered line
        driveSN=`echo $driveModelLine  | awk '/Model:/ {print $4}'`
    fi
    # either we have it now or not.
    if [ -z "$driveSN" ]; then
        return 1
    else
        echo $driveSN
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskSize(diskID)
#
#   Description:
#      get the disk size for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the disk size for
#
#   Outputs:
#       echo the disk size in mb
#
#   Returns:
#
#       0  (success) Disk size was detected and received.
#
#       1  (error) Disk size was NOT detected or received.
#
fn_eqlDiskSize()
{
    diskID="$1"
    # mib oid
    dvSize=`diskview -i $diskID |  awk 'BEGIN{IGNORECASE=1} /size=/, /blocks/ {print $1}' | awk -F"=" '{size=$2/2048;} END{ printf("%d", size) }'`
    if [ -z "$dvSize" ] || [ "$dvSize" -eq "0" ]; then
        return 1
    else
        echo $dvSize
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskStatus(diskID)
#
#   Description:
#      get the disk status for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the disk status for the requested disk id
#
#   Outputs:
#       the disk status
#
#   *NOTES:*
# +--------------------------------------------------------------------------------------------------------------------------
# |   - Disk status will be on-line when the disk is being used in the raidset.
# |   - Disk status will be spare when the disk is not currently being used by RAID configuration
# |       but may be used when status of a currently on-line disk changes.
# |   - Disk status will be failed when either it is faulted or when there is no power.
# |   - Disk status will be offline when the disk doesn't fall into any of these other categories.
# |   - Disk status will be too-small if drive is too small to be used. The drive cannot be converted to spare.
# |   - Disk status will be history-of-failures if drive has a history of failures. The drive can be converted to spare.
# |   - Disk status will be unsupported-version if drive label is from a later release. The drive can be converted to spare.
# |   - Disk status will be unhealthy if drive is unhealthy.
# |   - Disk status will be replacement if drive is a replacement drive.
# |   - Disk status will be encrypted if the drive is encrypted and cannot be decrypted.
# |   - Disk status will be notApproved if the drive is not DELL approved.
# |   - Disk status will be preempt-failed if the drive is off-line due to excessive errors.
# |
# |   *This set is not complete.*
# +--------------------------------------------------------------------------------------------------------------------------
#
#   Returns:
#
#       0  (success) Disk state was detected and received .
#
#       1  (error) Disk state was NOT etected and received .
#
fn_eqlDiskStatus()
{
    diskID="$1"
    # mib oid
    dState=`diskview -i ${diskID} | awk '/State:/ {state=NF; print $state}'`
    if [ -z "$dState" ]; then
        return 1
    else
        echo ${dState} | awk -f ./alib/tolower.awk
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskErrors(diskID)
#
#   Description:
#      get the disk errors for the requested disk id
#
#   Inputs:
#       _diskID_ the drive id to get the disk errors for the requested disk id
#
#   Outputs:
#       echo the count of disk errors since power on.
#
#   Returns:
#
#       0  (success) Disk error info was detected and received.
#
#       1  (error) Disk error info was NOT detected and received.
#
fn_eqlDiskErrors()
{
    diskID="$1"
    # mib oid
    eqlDiskId=.iso.org.dod.internet.private.enterprises.equalLogic.eqldiskModule.eqldiskObjects.eqlDiskTable.eqlDiskEntry.eqlDiskId
    eqlDiskErrors=iso.org.dod.internet.private.enterprises.equalLogic.eqldiskModule.eqldiskObjects.eqlDiskTable.eqlDiskEntry.eqlDiskErrors

    mibDiskIdVal=`fn_GetMib $eqlDiskId`
    mibDataVal=`fn_GetMib $eqlDiskErrors`
    mibDataVal=`fn_FilterModelLine "$mibDataVal"`
    mibVal=`echo start | awk -v DISKID=${diskID} -v eqlDiskId="${mibDiskIdVal}" -v eqlData="${mibDataVal}"  -f ./alib/eqlDiskEntry.awk`
    if [ -z "$mibVal" ]; then
        return 1
    else
        echo $mibVal
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskId(diskID)
#
#   Description:
#      get the disk ID for the requested disk id as seen by the subsystem
#
#   Inputs:
#       _diskID_ the drive id to get requested disk id as seen by the subsystem
#
#   Outputs:
#       echo the disk id as seen by the subsystem.
#
#   Returns:
#
#       0  (success) Disk id info was detected and received.
#
#       1  (error) Disk id info was NOT detected and received
#
fn_eqlDiskId()
{
    diskID="$1"
    # mib oid
    dRet=`diskview -i ${diskID} | awk '/^Disk/{print $2}'`
    if [ -z "$dRet" ]; then
        return 1
    else
        echo $dRet
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskSlot()
#
#   Description:
#      get the disk slot for the requested disk id as seen by the subsystem
#
#   Inputs:
#       _diskID_ the drive id to get the Slot information about
#
#   Outputs:
#       echo the disk slot number as marked on the back panel 0..N
#
#   Returns:
#
#       0  (success) Disk slot info was detected and received.
#
#       1  (error) Disk slot info was NOT detected and received
#
fn_eqlDiskSlot()
{
    diskID="$1"
    # mib oid
    eqlDiskId=.iso.org.dod.internet.private.enterprises.equalLogic.eqldiskModule.eqldiskObjects.eqlDiskTable.eqlDiskEntry.eqlDiskId
    mibDiskIdVal=`fn_GetMib $eqlDiskId`
    mibVal=`echo start | awk -v DISKID=${diskID} -v eqlDiskId="${mibDiskIdVal}" -v eqlData="${mibDiskIdVal}"  -f ./alib/eqlDiskEntry.awk`
    if [ -z "$mibVal" ]; then
        return 1
    else
        echo $mibVal
        return 0
    fi
}

# ------------------------------------------
#   Function:   fn_eqlDiskTypeEnum(diskID)
#
#   Description:
#      get the disk type for the requested disk id as seen by the subsystem
#
#    _ENUM_       _Value_
#    - unknown      0
#    - sata         1
#    - sas          2
#    - sata-ssd     3
#    - sas-sed-hdd  5
#    - sas-sed-ssd  6
#
#   Inputs:
#       _diskID_ the drive id to get the type information abou
#
#   Outputs:
#       echo the enum type of this disk(see description)
#
#   Returns:
#
#       0  (success) Disk disk enum type info was detected and received.
#
#       1  (error) Disk disk enum type info was NOT detected and received
#
fn_eqlDiskTypeEnum()
{
    diskID="$1"
    # mib oid
    eqlDiskId=.iso.org.dod.internet.private.enterprises.equalLogic.eqldiskModule.eqldiskObjects.eqlDiskTable.eqlDiskEntry.eqlDiskId
    eqlDiskTypeEnum=.iso.org.dod.internet.private.enterprises.equalLogic.eqldiskModule.eqldiskObjects.eqlDiskTable.eqlDiskEntry.eqlDiskTypeEnum

    mibDiskIdVal=`fn_GetMib $eqlDiskId`
    mibDataVal=`fn_GetMib $eqlDiskTypeEnum`
    mibDataVal=`fn_FilterModelLine "$mibDataVal"`
    mibVal=`echo start | awk -v DISKID=${diskID} -v eqlDiskId="${mibDiskIdVal}" -v eqlData="${mibDataVal}"  -f ./alib/eqlDiskEntry.awk`
    if [ -z "$mibVal" ]; then
        return 1
    else
        echo $mibVal
        return 0
    fi
}

####################
## Misc rtns

# ------------------------------------------
#   Function:       fn_GetDrivesInAllRaidsets()
#
#   Description:
#      Get a sorted list of drives in a Raidset. The user should check that the
#      raidset exists before calling this method
#
#   Inputs:
#       none
#
#   Outputs:
#       An ascending list of drive id'.
#
#   Returns:
#       0  (success) Disk drives in the raidsets were detected and received.
#
#       1  (error)  Disk drives in the raidsets were NOT detected and received.
#
fn_GetDrivesInAllRaidsets()
{
    # now run the output of 'raidtool -Z' against that script
    drives=`raidtool -Z | awk -f ./alib/sortRaidDrives.awk `
    if [ -z "$drives" ]; then
        fn_Log "    Unable to obtain a list of drives in the raidset."
        return 1
    fi
    echo $drives
    return 0
}

# ------------------------------------------
#   Function:       fn_CheckForSuspectDrive( driveToCheck )
#
#   Description:
#      Check whether a disk is a suspect drive with  recognized firmware versions
#
#   Inputs:
#       _driveToCheck_  The drive ID to check.
#
#   Outputs:
#       A warning message when an unusual suspect drive firmware is detected
#
#   Returns:
#       0  (success) when a suspect Drive is detected
#
#       1  (error)  when a suspect Drive is NOT detected
fn_CheckForSuspectDrive()
{

    driveToCheck=$1

    eqlSuspectDriveDetected=1

    driveType=`fn_eqlDiskModelNumber $driveToCheck`
    driveFwRev=`fn_eqlDiskRevisionNumber $driveToCheck`
    driveSn=`fn_eqlDiskSerialNumber $driveToCheck`
    # see if we have a descriptor for this drives model
    if [ -f  "./suspects/${driveType}.dsc" ]; then
        # check for wildcard
        wildCardFound=`cat ./suspects/${driveType}.dsc | awk -v SN=ALL -f ./alib/driveSerialMatcher.awk`
        if [ ! -z "$wildCardFound" ]; then
            fn_Log "$wildCardFound"
            eqlSuspectDriveDetected=0
        fi
        # check for specific serial number
        snFound=`cat ./suspects/${driveType}.dsc | awk -v SN=$driveSn  -f ./alib/driveSerialMatcher.awk`
        if [ ! -z "$snFound" ]; then
            fn_Log "$snFound"
            eqlSuspectDriveDetected=0
        fi
    fi

    return $eqlSuspectDriveDetected
}

# ------------------------------------------
#   Function:       fn_CountSuspectDrives()
#
#   Description:
#      Method to count the number of Online suspect drives present in the array
#
#   Inputs:
#       none
#
#   Outputs:
#       the integer count of suspect drives is echoed to stdout
#
#   Returns:
#       0  (success)  Suspect drives were detected and counted.
#
#       1  (error)   Data used to detect suspect drives was invalid.
#
fn_CountSuspectDrives ()
{
    allDrives=`diskview -I s`
    fn_Log "fn_CountSuspectDrives"  'T'
    fn_Log "$allDrives"  'T'
    onlineDrive=`diskview -I s | awk 'BEGIN { IGNORECASE = 1 } /Online|Scanning/ { print $2 }'`
    if [ $? -ne 0 ]; then
        return 1
    fi

    x=0
    for f in $onlineDrive
    do
        SNmatch=`fn_CheckForSuspectDrive $f`
        if [ $? -eq 0 ]; then
            x=`echo $x 1 | awk '{printf("%d", $1+$2)}'`
        fi
    done
    echo $x
    return 0
}
#   Function:       fn_detectSmartExceededThresholdDrives()
#
#   Description:
#      Check for SATA drives in raidset that have exceeded the smart threshold
#
#   Inputs:
#       none
#
#   Outputs:
#       - the SATA driveID of the first drive in the raidset that has exceeded
#         smart threshold is sent to stdout
#
#   Returns:
#           0  (success) Drive that has exceeded smart threshold is detected
#
#           1  (error) Drive that has NOT exceeded smart threshold is detected
#
fn_detectSmartExceededThresholdDrives()
{
    DRIVES=`fn_GetDrivesInAllRaidsets`
    for d in $DRIVES
    do

        # check for a drive in the raidset that has EXCEEDED its smart threshold
        isSata=`diskview -i $d | awk '/SATA\|/'`
        exceeded=`diskview -F $d  | awk '/EXCEEDED/'`
        if [ ! -z "$exceeded" ]   && [ ! -z "$isSata" ]; then
            echo $d
            return 0
            break;
        fi
    done
    return 1
}
# ------------------------------------------
#   Function:       fn_GetSmart(driveID diskviewOption)
#
#   Description:
#      Log diskview output of 3 possible option -B -T  and -F
#   Inputs:
#       _driveID_    the drive id to get the diskview output for
#
#       _diskviewOption_  One of 3 possible options(see description)
#
#  Outputs:
#       The output of diskview OR a message indicating that the data was unavailable.
#
#   Returns:
#       0 Requested Output was successfuly obtained
#       1 unable to obtain requested output
#

fn_GetSmart()
{
    driveID=$1
    diskviewOption=$2

    smartDeviceData=""
    tries=0
    maxTries=6
    ######
    driveStatus=`diskview -i $driveID | awk '/State:/ {print $7 $8}' 2>&1`
    if  [ "$driveStatus" = "online" ] || [ "$driveStatus" = "Online" ]; then
        # this is our marker var.
        RSC=""
        while [ -z "${smartDeviceData}" ]
        do
            sleep 2   # deep cleansing breaths...
            smartDeviceData=`diskview -${diskviewOption} ${driveID}  2>&1`
            pdioctlHit=`echo "${smartDeviceData}" | awk 'BEGIN{IGNORECASE=1} /ioctl/'`
            if [ ! -z "${pdioctlHit}" ];then
                tries=`echo $tries | awk '{print $0 + 1}' `
                smartDeviceData=""
            else
                if [ "${diskviewOption}" = "F" ]; then
                    RSC=`echo "${smartDeviceData}" | awk 'BEGIN{IGNORECASE=1} /EXCEEDED/ {print $0} ' 2>&1`
               fi
            fi

            # well do we try again
            if [ $tries -gt $maxTries  ]; then
                fn_Log "###################################################" "T"
                fn_Log "    Drive ${driveID}  Smart Device Data unavailable" "T"
                fn_Log "###################################################" "T"

                return 1
            fi
        done
        # We got it, so choose the appropriate output
        if [ "${diskviewOption}" = "F" ]; then
            fn_Log "    Drive ${driveID} ${RSC}" "T"
        fi

        if [ "${diskviewOption}" = "B"  -o "${diskviewOption}" = "T" ]; then
            fn_Log "###################################################" "T"
            fn_Log " Drive ${driveID}"           "T"
            fn_Log "####################"        "T"
            fn_Log "${smartDeviceData}"          "T"
        fi
        fn_Log "----------------------------------------------------------------------" "T"
        return 0
    else
        fn_Log "################################################################"       "T"
        fn_Log "    Drive ${driveID} is not online Smart Device Data unavailable"       "T"
        fn_Log "################################################################"       "T"
        fn_Log "----------------------------------------------------------------------" "T"
        return 1
    fi
    # should never hit this
    return 0
}
