#!/bin/sh
#
# @(#) Script to perform a raidtool mirror copy of drives allowing for a new
# set of hard harddrives to be rotated onto an array as substitutes
#
# Copyright (c) 2002-2017 by Dell Inc.
#
# All rights reserved.  This software may not be copied, disclosed,
# transferred, or used except in accordance with a license granted
# by Dell Inc.  This software embodies proprietary information
# and trade secrets of Dell Inc.
#
. ./lib/commonLib.sh


# ------------------------------------------
#   Function:       fn_GetRaidLunContainingDiskID(diskID)
#      Function to return the RAID LUN containing the diskid the user wants to work with
#
#   Inputs:
#       _diskID_
#
#   Outputs:
#   - the RAID Lun containing the diskid input
#
#   return:
#       0  (success) RAID lun was found for specified diskID
#
#       1  (error) RAID lun was NOT found for specified diskID
#
fn_GetRaidLunContainingDiskID()
{
    diskID="$1"
    if [ -z "$diskID" ]; then
        return 1
    fi

    retRaidLun=`raidtool | awk -v DISKID=$diskID -v IWANT=lun -f ./alib/findLunInfoForDiskId.awk`

    if [ -z "$retRaidLun" ]; then
        return 1
    fi
    echo "$retRaidLun"
    return 0

}

# ------------------------------------------
#   Function:       fn_GetRaidType([raidLun])
#      Function to return the raidtype of a raid LUN
#
#   Inputs:
#       _raidLun_ or _nothing_ (default is lun 0)
#
#   Outputs:
#        the RAID type of the requested LUN is output(lowercase)
#
#   return:
#       0  (success) RAID type was detected
#
#       1  (error) RAID type was NOT detected
#
fn_GetRaidType()
{
    raidLun="$1"
    if [ -z "$raidLun" ]; then
        raidLun=0
    fi
    retRaidType=`raidtool |  awk  -v LUN=$raidLun -f ./alib/multiLun.awk | awk -f ./alib/raidType.awk`

    if [ -z "$retRaidType" ]; then
        return 1
    fi
    echo "$retRaidType"
    return 0

}


# ------------------------------------------
#   Function:       fn_PerformMirrorCopy()
#      Initiate a raidtool mirror copy to/from explicit drives.
#
#   Inputs:
#       _fromDrive_ the disk to perform the raidtool mirror copy "from"
#
#       _toDrive_  the spare disk to perform the raidtool mirror copy "to"
#
#       _noFail_  Optional argument that is passed will add the '-noFail' option to the mirror copy
#
#   Outputs:
#       none
#
#   Returns:
#       0  (success) when the raidtool mirror copy successfully initiated
#
#       1  (error) when the raidtool mirror copy failed to initiate
#
fn_PerformMirrorCopy()
{
    fromDrive=$1;
    toDrive=$2;
    noFail="$3"
    # if passed anything then add '-noFail' switch to the raidtool mirror initiate
    if [ ! -z "$noFail" ]; then
        noFail="-noFail"
    fi
    fn_Log "    Performing a copy-to-spare from drive $fromDrive to spare $toDrive"
    # enable mirroring
    enableMessage=`raidtool -m enable`
    fn_Log "    $enableMessage" "T"
    disableMessage=`raidtool -m rebuild_disable`
    fn_Log "    $disableMessage" "T"

    initiateMessage=`raidtool -m $fromDrive initiate -copyTo=$toDrive ${noFail} `
    mirrorInitiated=$?

    errorCheck=`echo $initiateMessage | awk '/ioctl/'`
    if [ ! -z "$errorCheck" ]; then
        fn_Log "    $errorCheck"
        return 1
    fi
    fn_Log "    [ $initiateMessage ]" "T"
    sleep 5
    status=`raidtool | awk '/Mirror of source drive/'`
    if [ -z "$status" ]; then
        return 1
    fi
    return 0
}
# ------------------------------------------
#   Function:       fn_CancelMirrorCopy()
#      Cancel the current Mirror copy operation.
#
#   Inputs:
#       none
#
#   Outputs:
#       The results of the command are logged.
#
#   Returns:
#       0   (success) Mirror copy was canceled
#
#       1   (error) Mirror copy was NOT canceled
#
#
fn_CancelMirrorCopy()
{
    output=`raidtool`
    drive=`echo $output | awk '/Mirror of source drive/ {print $5 }'`
    if [ -z "$drive" ]; then
        echo ""
        fn_Log "        Warning: no copy-to-spare operation is in progress on this array"
        return 1
    fi
    output=`raidtool -m $drive cancel`
    fn_Log "$output"  "T"
    echo ""
    fn_Log "        INFO: Copy-to-spare canceled"
    return 0
}
# ------------------------------------------
#   Function:       fn_GetMirrorCopyStatus()
#      Check the progress of a raidtool mirror copy
#
#   Inputs:
#       none
#
#   Outputs:
#       A message when the RAID mirror copy is not running
#
#   Returns:
#           0  (success) when the raidtool mirror copy is running
#
#           1  (error) when the raidtool mirror copy is not running
#
fn_GetMirrorCopyStatus()
{
    mstatus=`raidtool | awk '/Mirror of source drive/'`
    if [ -z "$mstatus" ]; then
        fn_Log "    RAID copy-to-spare is NOT in progress" "T"
        return 1
    fi
    fn_Log "    $mstatus"
    return 0
}
# ------------------------------------------
#   Function:       fn_GetRaidStatus()
#      Check the status of the RAID on an array
#
#   Inputs:
#       none
#
#   Outputs:
#       A warning message when raidset is not ready or not healthy
#
#   Returns:
# +--------------------------------------------------------------------------------------------------------------------------
# |  - 0 raidset is in a proper state
# |  - 1 when the raidset is expanding
# |  - 2 when the raidset is verifying
# |  - 3 when the raidset is reconstructing
# |  - 4 when the raidset is degraded
# |  - 5 when the raidset lun(s) are not marked 'Ok'
# |  - 6 when the  drive has a history of failures and is unavailable.
# |  - 7 when there is a mirror copy in progress
# |  - 8 when the array has not been setup
# +--------------------------------------------------------------------------------------------------------------------------
#
fn_GetRaidStatus()
{
    # Note the order we check these is important.
    fn_Log "    Checking the raidset status on all Lun's"
    # Check that none of the conditions that would prevent the raidtool mirror copy to fail currently exists.
    #
    # Normally we do not proceed if the array has not been setup.(added for completeness)
    arrayIsNotSetup=`raidtool -V 0| awk '/array has not been setup/'`
    if [ ! -z "$arrayIsNotSetup" ]; then
        fn_Log "    WARNING: The array has not been setup"
        return 8
    fi
    #  Normally we do not proceed if the raidset is currently reconstructing
    reconstruction=`raidtool | awk '/Reconstruction/'`
    if [ ! -z "$reconstruction" ]; then
        fn_Log "    WARNING: RAID Reconstruction is in progress "
        return 3
    fi
    # Normally we do not proceed if the raidset is currently Degraded
    degraded=`raidtool | awk '/Degraded/'`
    if [ ! -z "$degraded" ]; then
        fn_Log "    WARNING: RAID is Degraded"
        return 4
    fi
    # Normally we do not proceed if any of the drives are unavailable.
    diskUnavailable=`raidtool | awk '/history/'`
    if [ ! -z "$diskUnavailable" ]; then
        fn_Log "    WARNING: Not all drives are available"
        return 6
    fi
    # Normally we do not proceed if the raidset is expanding
    expansion=`raidtool | awk '/Expansion/'`
    if [ ! -z "$expansion" ]; then
        fn_Log "    WARNING: RAID Expansion is in progress "
        return 1
    fi
    # Normally we do not proceed if there is a RAID mirror copy already in progress.
    mstatus=`raidtool | awk '/Mirror of source drive/'`
    if [ ! -z "$mstatus" ]; then
        fn_Log "    WARNING: RAID mirror is currently running"
        return 7
    fi
    # Normally we do not proceed if all the RAID LUN's are not reporting a status of 'Ok'
    raidLunCount=`raidtool | awk '/RAID LUN/{count++}; END{print count}'`
    raidLunOkCount=`raidtool | awk '/RAID LUN/' | awk 'BEGIN{count=0}/Ok/{count++}; END{print count }'`
    if [ $raidLunCount -ne $raidLunOkCount ]; then
        fn_Log "    WARNING: Not all RAID LUN are reporting 'Ok'"
        return 5
    fi
    #
    ##  Normally we do not proceed if the raidset is currently verifying
    # but sometimes we do, therefore we do this check LAST
    verification=`raidtool | awk '/Verification/'`
    if [ ! -z "$verification" ]; then
        fn_Log "    WARNING: RAID Verification is in progress "
        return 2
    fi

    # We must be ok to proceed.
    return 0
}

# ------------------------------------------
#   Function:       fn_GetRaidLunCount()
#      Function to return the number of Luns on a member
#
#   Inputs:
#       none
#
#   Outputs:
#   the  number of luns found
#
#   return:
#       0  (success) Number of luns was detected
#
#       1  (error) Number of luns was not detected
#
fn_GetRaidLunCount()
{

    lunCount=`raidtool | awk 'BEGIN {lcount=0} /RAID LUN/ {lcount++} END{print lcount}'`
    if [ -z "$lunCount" ]; then
        return 1
    fi
    echo "$lunCount"
    return 0
}
# ------------------------------------------
#   Function:       fn_GetRaidsetDrives( [raidLun] )
#      Function to return the list of  drives in a raidset LUN
#
#   Inputs:
#       _raidLun_ or nothing (default is lun 0)
#
#   Outputs:
#   - space separated list drives
#
#   return:
#       0  (success) Drives in raidset lun were detected
#
#       1  (error) Drives in raidset lun were NOT detected
#
fn_GetRaidsetDrives()
{
    raidLun="$1"
    if [ -z "$raidLun" ]; then
        raidLun=0
    fi
    retDrives=`raidtool | awk  -v LUN=$raidLun -f ./alib/multiLun.awk | awk -f ./alib/raidsetDrives.awk`
    if [ -z "$retDrives" ]; then
        return 1
    fi
    echo "$retDrives"
    return 0
}



# ------------------------------------------
#   Function:       fn_GetRaidSpares()
#      Output the list of spare drives
#
#   Inputs:
#       none
#
#   Outputs:
#       The space delimited list of spares is output -OR- a warning message
#
#
#   Returns:
#       0   - successfully found spares and output(see above ) contains list
#
#       1   - failed to find spares and output(see above ) contains warning
#
fn_GetRaidSpares()
{
    # check that there are indeed spares available
    spares=`raidtool -Z | awk '/hot spare/{gsub(/lun=/, "", $0); print $2}'`
    if [ -z "$spares" ]; then
        fn_Log "    Warning: no spare drives are available"
        return 1
    fi
    echo $spares |awk '{ sub(/\n$/," "); print }'
    return 0
}

# ------------------------------------------
#   Function:       fn_GetLunInfoForDiskId( diskID, raidInfoWanted )
#      Function to find the RAID information the user wants for a particular diskID.
#
#   Inputs:
#       _diskID_  the disk id the user is interested in
#
#       _raidInfoWanted_  the RAID information for a disk ID the user wants.
#                       choices are LUN, DRIVES, RAIDTYPE or CAPACITY.
#   Outputs:
#   - the raidInfoWanted data is echoed to stdout
#
#   return:
#       0  (success) RAID information retrieved for disk id
#
#       1  (error) RAID information was NOT retrieved for disk id
#
fn_GetLunInfoForDiskId()
{
    diskID="$1"
    raidInfoWanted="$2"

    #  get the info the user asked for
    raidInfoRet=`raidtool | awk -v DISKID=$diskID -v IWANT=$raidInfoWanted  -f ./alib/findLunInfoForDiskId.awk`
    if [ -z "$raidInfoRet" ]; then
        return 1
    fi
    echo "$raidInfoRet"
    return 0
}


# ------------------------------------------
#   Function:       fn_GetRaidsetForDiskId( diskID )
#      Function to find the raidset containing a particular diskID.
#
#
#   Inputs:
#       _diskID_  the disk id the user is interested in
#
#   Outputs:
#   - the raidset containing the diskID is echoed to stdout
#
#   return:
#       0  (success) raidset containing disk id was found
#
#       1  (error) raidset containing disk id was NOT found
#
fn_GetRaidsetForDiskId()
{
    diskID="$1"


    #  get the info the user asked for
    raidsetRet=`raidtool | awk -v DISKID=$diskID  -f ./alib/findRaidsetForDiskId.awk`
    if [ -z "$raidsetRet" ]; then
        return 1
    fi
    echo "$raidsetRet"
    return 0
}

# ------------------------------------------
#   Function:       fn_GetAdjacentDriveForDiskId(diskID, whichSide, diskLBA)
#      Function to find the raidset containing a particular diskID.
#
#   Inputs:
#       _diskID_  the disk id the user is interested in
#
#       _whichSide_  the drive on the "leftside" or "rightside" adjacent to the diskID
#
#       _diskLBA_  The disk lba used to check if this is a parity or user data block.
#
#   Outputs:
#   - the driveID of the adjacent drive is echoed.
#
#   return:
#       0  (success) Adjacent drive on requested side was found
#
#       1  (error) Adjacent drive on requested side was NOT found
#
#       2 - success but adjacent drive on requested side is parity
#
fn_GetAdjacentDriveForDiskId()
{
    diskID="$1"
    whichSide="$2"
    diskLBA="$3"

    #  get the info the user asked for
    adjacentDrive=`raidtool | awk -v DISKID=$diskID -v IWANT=$whichSide -f ./alib/findAdjacentDriveForDiskId.awk`
    if [ -z "$adjacentDrive" ]; then
        return 1
    fi
    # check if the adjacent drive is a parity drive and if so get the next adjacent drive in the requested direction.
    parityBlock=`raidtool -O $adjacentDrive -M $diskLBA | awk '/[pP]arity/ {print $0}'`
    fn_Log "ParityBlock => $parityBlock" "T"
    if [ ! -z "$parityBlock" ]; then
        fn_Log "    Adjacent drive: $adjacentDrive found on $whichSide of $diskID is a Parity Drive" "T"
        fn_Log "    We will now look to the $whichSide of that parity block drive: $adjacentDrive" "T"
        # go get the next adjacent drive.
        adjacentDrive=`raidtool | awk -v DISKID=$adjacentDrive -v IWANT=$whichSide -f ./alib/findAdjacentDriveForDiskId.awk`
        if [ -z "$adjacentDrive" ]; then
            return 1
        fi
        # we can echo the driveID for any checking however the result has to indicate that we skipped the parity drive block
        echo "$adjacentDrive"
        return 2
    fi
    echo "$adjacentDrive"
    return 0
}

# ------------------------------------------
#   Function:       fn_GetRaidsetDriveCount([raidLun])
#      Function to return the list of spares on an array
#
#   Inputs:
#       _raidLun_ or _nothing_ (default is lun 0)
#
#   Outputs:
#      number of drives in the raidset for the specified LUN
#
#   return:
#       0  (success) List of spares was found
#
#       1  (error) List of spares was NOT found
#
fn_GetRaidsetDriveCount()
{
    raidLun="$1"
    if [ -z "$raidLun" ]; then
        raidLun=0
    fi
    retDriveCount=`raidtool | awk  -v LUN=$raidLun -f ./alib/multiLun.awk | awk -f ./alib/raidsetDriveCount.awk`
    if [ -z "$retDriveCount" ]; then
        return 1
    fi
    echo "$retDriveCount"
    return 0
}

# ------------------------------------------
#   Function:       fn_GetRaidsetFailedDrives([raidLun])
#      Function to return the list of  drives in a raidset that are failed
#
#   Inputs:
#       _raidLun_ or _nothing_ (default is lun 0)
#
#   Outputs:
#    list of drives rebuilding in a raidset lun
#
#   return:
#       0  (success) failed drives were found
#
#       1  (error) drives were NOT found
#
fn_GetRaidsetFailedDrives()
{

    rSetDrives=`raidtool | awk -v IWANT=fail -f ./alib/raidDriveWithCondition.awk`
    if [ -z "$rSetDrives" ]; then
        return 1
    fi

    echo "$rSetDrives"
    return 0
}
# ------------------------------------------
#   Function:       fn_GetAllRaidsetReconDrives()
#      Function to return the list of  drives for all raidset LUN's that are rebuilding
#
#   Inputs:
#       none
#
#   Outputs:
#    list of drives rebuilding in a raidset lun
#
#   return:
#       0 - drives rebuilding were found
#
#       1 - drives rebuilding were NOT found
#
fn_GetAllRaidsetReconDrives()
{
    rSetDrives=`raidtool | awk -v IWANT=recon -f ./alib/raidDriveWithCondition.awk`
    if [ -z "$rSetDrives" ]; then
        return 1
    fi

    echo "$rSetDrives"
    return 0
}

# ------------------------------------------
#   Function:       fn_ReverseLbaLookup( diskID, diskLBA )
#      Function to find the block on disk that tells us the corresponding
#      raidset information
#
#   Inputs:
#   _diskID_ The disk id of the drive to lookup
#
#   _diskLBA_ The Disk lba on the drive to lookup
#
#   Outputs:
#     A 2 part string containing values for the RAID LUN and the RAID LBA
#     that correspond to the input args for the diskID and the diskLUN.
#     OR the error output from raidtool
#
#   return:
#       0  (success) RAID lun and raid lba were detected
#
#       1  (error) RAID lun and raid lba were not detected
#
#       2  (error) RAID lun/lba are a parity block
#
fn_ReverseLbaLookup()
{
    diskID="$1"
    diskLBA="$2"
    # capture the output in case of error
    fn_Log "raidtool -O $diskID -M $diskLBA" "T"
    rawLookup=`raidtool -O $diskID -M $diskLBA`
    parityBlock=`raidtool  -O $diskID -M $diskLBA | awk -v IWANT=parity -f ./alib/raidReverseLookup.awk`
    if [ ! -z "$parityBlock" ]; then
        fn_Log "    Reverse Lookup for Drive $diskID and diskLBA $diskLBA returns a Parity Block"
        fn_Log "    parityBlock=> $parityBlock"
        echo "$rawLookup"
        return 2
    fi
    # do the lookup a second time for real
    retLUN=`raidtool  -O $diskID -M $diskLBA | awk -v IWANT=lun -f ./alib/raidReverseLookup.awk`
    if [ -z "$retLUN" ]; then
        fn_Log "retLUN is empty "
        echo "$rawLookup"
        return 1
    fi
    retLBA=`raidtool  -O $diskID -M $diskLBA  | awk -v IWANT=lba -f ./alib/raidReverseLookup.awk`
    if [ -z "$retLBA" ]; then
        fn_Log "retLBA is empty "
        echo "$rawLookup"
        return 1
    fi
    echo "LUN:$retLUN"
    echo "LBA:$retLBA"
    return 0
}

# ------------------------------------------
#   Function:       fn_LostBlockPuncture( raidLUN, raidLBA )
#      Function to sets  a RAID  lba as lost block on a raid lun
#
#   Inputs:
#   _raidLUN_ The RAID lun containing the RAID lba
#
#   _raidLBA_ The RAID lba on the  lun to set as a lost block
#
#   Outputs:
#    the actual output of the command
#
#   return:
#       0  (success) lost block puncture succeeded
#
#       1  (error) lost block puncture failed
#
fn_LostBlockPuncture()
{
    raidLUN="$1"
    raidLBA="$2"
    blockCount="$3"
    # check optional arg.
    if [ ! -z "$4" ]; then
        ECHO_MODE="$4"
    else
        ECHO_MODE="0"
    fi

    fn_Log "    Performing Lost Block Puncture on RAID LUN: $raidLUN LBA: $raidLBA for $blockCount Blocks"
    # calculate the ending LBA using the block count as the offset
    endRaidLBA=`echo $raidLBA $blockCount | awk '{ print $1+$2 }'`
    while [ $raidLBA -lt $endRaidLBA ]
    do
        # Log the command we are about to invoke
        fn_Log "raidtool -w $raidLUN -M $raidLBA"
        if [ $ECHO_MODE -eq 0 ]; then
            lbpOut=`raidtool -w $raidLUN -M $raidLBA`
            if [ $? -eq 1 ]; then
                fn_Log "$lbpOut"
               return 1
            fi
            echo "$lbpOut"
        fi
        # increment the lba
        raidLBA=`echo $raidLBA | awk '{ print $1+1}'`
    done
    fn_Log "Completed Lost Block Puncture..."
    return 0
}
# ------------------------------------------
#   Function:       fn_DD_ZERO( driveID, diskLBA, blockCount, [echoMode] )
#      Function to Zero the blocks from the DISK_LBA thru the range represented by BLOCK_COUNT on a drive
#
#   Inputs:
#    _diskID_ The drive ID of the disk to zero
#
#    _diskLBA_ The RAID lba on the  lun to zero
#
#    _blockCount_ The number of blocks to zero
#
#    _echo mode_ Optional arg (if set) to only echo and not actually execute the command
#                 the default is false.
#
#   Outputs:
#   - the command line is echoed
#
#   return:
#       0  (success) dd command execution succeeded
#
#       1  (error) dd command execution failed
#
fn_DD_ZERO()
{
    driveID="$1"
    diskLBA="$2"
    blockCount="$3"
    # check optional arg.
    if [ ! -z "$4" ]; then
        ECHO_MODE="$4"
    else
        ECHO_MODE="0"
    fi
    # Zero the blocks from the diskLBA thru the range represented by blockCount
    fn_Log "    Performing ZEROing operation"
    fn_Log "      dd if=/dev/zero of=/dev/rpd${driveID}d bs=512 count=${blockCount} seek=${diskLBA}"
    if [ $ECHO_MODE -eq 0 ]; then
        dd if=/dev/zero of=/dev/rpd${driveID}d bs=512 count=${blockCount} seek=${diskLBA}
        if [ $? -ne 0 ]; then
            ENDBLOCK=`echo ${diskLBA} ${blockCount} | awk '{ print $1 + ($2 - 1)}'`
            return 1
        fi
    fi
    echo ""
    return 0
}
